## Code used for paper "Modeling a hot, dry future: substantial range reductions in suitable environment projected under climate change for a semiarid riparian predator guild", performed in R v 4.1.1
## Authors: Blais, BR and JL Koprowski
## Correspondence: bblais@arizona.edu
##
######################################
#
## *Notes
## 
## components of this project required downloading very large files elsewhere (e.g., bioclimatic variables from WorldClim 2.0)  
## and processing/clipping spatial extent of rasterized bioclimatic predictor to the study area and in ArcGIS (ArcMap). Therefore, 
## we called large files stored on local machines; users will need to acquire rasters as described in the paper's Methods section 
## or download from packages (see lines 191). Also, some of the focal taxa are threatened under the Endangered Species Act and their
## occurrences cannot be shared. Those data inquiries can be directed to Arizona Game and Fish Department, Heritage Data Management 
## System (HDMS) Program Manager at 623 236-7618 or HDMS@azgfd.gov. Additional inquiries can be directed to the corresponding author. 
## Data for this code are stored in Supplementary file Dataset S1 (xlsx), sheets correspond to dataframes used here (save sheets as csv files)
#
#
#######################################


getwd() # check working directory


## Create vector of helpful/required packages
SDM.packages<-c("biomod2","dismo","geodata","ggpubr","gridExtra","psych","raster","rasterVis","reshape2","rgdal",
                "spThin","terra","tictoc","tidyr","tidyterra","tidyverse"
                )

# Load package bundle
lapply(SDM.packages, require, character.only = TRUE)    


#####################
#set up colorblind-friendly palette
cbPalette <- c("#E69F00", "#56B4E9", "#009E73", "#CC79A7", "#0072B2", "#D55E00", "#999999","#F0E442","lightgray","#000000" )


#######################

#load datasets
THAM.df<-read.csv("THAM_share-copy.csv",sep=",", header=T) #csv dataset with geocoordinates; *excludes sensitive records of protected species (THEQ,THRU)

rownames(THAM.df)<-THAM.df[,1] #tell R to use column 1 as row names
THAM.df[,1]<-NULL #nullifies row 1 as a variable (when arbitrary column is used)


#Factor conversion for THAM.df dataset
THAM.df$period = factor(THAM.df$period,
                        ordered = FALSE,
                        levels=c("a","b","c"),
                        labels=c("pre-1940","1940-1979","1980-current")) #creates easier labels for time periods

THAM.df$species = factor(THAM.df$species,
                        ordered = FALSE,
                        levels=c("THCY","THEL","THMA"), #4-letter taxa codes
                        labels=c("Thamnophis cyrtopsis","Thamnophis elegans","Thamnophis marcianus")) #full sci names

#set categorical factors en masse 
THAM.df.cols <- c("source","county")  
THAM.df[,THAM.df.cols] <- lapply(THAM.df[,THAM.df.cols], factor)

THAM.df$dmy = as_date(dmy(THAM.df$dmy)) #

str(THAM.df)  
############################

## explore data

table(THAM.df$species)
  #THCY THEL THEQ THMA THRU 
  # 251  220   42  123  499 

table(THAM.df$species,THAM.df$period)
  #     pre-1940 1940-1979 1980-current
  #THCY       17        54          180
  #THEL       13        37          170
  #THEQ        5        11           26
  #THMA        3        37           83
  #THRU        2        21          476

#
#
######################################################################

## Prepare data to use only current (i.e., present) records (1980-2021) and partition species

#all THAM,  1980 current
THAM<-dplyr::filter(THAM.df,period=="1980-current")%>% 
  select(species,ddlon,ddlat)
rownames(THAM)<-THAM[,1] #tell R to use column 1 as row names
THAM[,1]<-NULL #nullifies row 1 as a variable (when arbitrary column is used)

#species subsets
THCY<-dplyr::filter(THAM.df,species=="THCY" & period=="1980-current")%>% #THCY Only,  1980 current
  select(ddlon,ddlat,species) 
THEL<-dplyr::filter(THAM.df,species=="THEL" & period=="1980-current")%>% #THEL Only,  1980 current
  select(ddlon,ddlat,species)  
THMA<-dplyr::filter(THAM.df,species=="THMA" & period=="1980-current")%>% #THMA Only,  1980 current
  select(ddlon,ddlat,species)

  ## Protected species omitted ##
    # THEQ<-dplyr::filter(THAM.df,species=="THEQ" & period=="1980-current")%>% #THEQ Only,  1980 current
    #   select(ddlon,ddlat,species) 
    # THRU<-dplyr::filter(THAM.df,species=="THRU" & period=="1980-current")%>% #THRU Only,  1980 current
    #   select(ddlon,ddlat,species)  

#########################################
## Prelim  corr check of env vars 
 # *requires local files for bioclim variables (.tif) and spatial frame extents

  Arizona<-readOGR(dsn="./Spatial/USA", layer = "ArizonaWGS84") #local file, else call online/package source (datum=WGS84)

  plot(Arizona) #quick look at spatial frame
  AZ_counties<-readOGR(dsn="./Spatial/USA/county", layer = "AZ_counties") #adding counties for visual convenience
  
  AZpath<-file.path(".","Spatial","WORLDCLIM","AZcrop",fsep="/") #enter local drive and folders in "" 
  AZworldclim<-list.files(AZpath,pattern='tif$',full.names=TRUE) #call all ".tif" files from local device
  print(AZworldclim)
  
  ## Initial exploration of occurrences/background against predictors; i.e., steps to reduce colinearity [for helpful guidance, see https://rspatial.org/sdm/4_sdm_envdata.html]
  #predictors <- stack(worldclim)
  predictors <- stack(AZworldclim) #raster package
  head(predictors) #note order (1,10-17,3-6,elev)
  names(predictors)<-c("bio1","bio10","bio11","bio12","bio13","bio14","bio15","bio16","bio17","bio3","bio4","bio5","bio6","elev")
  plot(predictors) #all
  
  plot(predictors,2) #just nth predictor
  plot(AZ_counties,add=TRUE)
  points(THCY, col="red") #add points, THCY for example
  
  presvalsTHAM <- raster::extract(predictors, THAM)
  backgrTHAM <- randomPoints(predictors, 900) #900 random background pts (i.e., 5-fold increase of abundant sp [180 THCY]), *note random pts will be within spatial extent of "predictors",i.e., Arizona here
  absvalsTHAM <- raster::extract(predictors, backgrTHAM)
  pbTHAM <- c(rep(1, nrow(presvalsTHAM)), rep(0, nrow(absvalsTHAM))) #pb = present OR background
  sdmdataTHAM <- data.frame(cbind(pbTHAM, rbind(presvalsTHAM, absvalsTHAM)))
  
  head(sdmdataTHAM)
  
  pairs(sdmdataTHAM[,2:14], cex=0.1) #matrix plots~ visualize colinearity 
  
  #Correlation matrix 
  corr.test(sdmdataTHAM[,2:14],
            use = "pairwise",
            method="spearman",
            adjust="none",      
            alpha=.05) #

#
#
########################################

## Thinning individual species occurrences to 1 per sq km
# (perform for each unique species)

tic() #package tictoc provides time elapsed to run command(s)
THCY_thin<-thin(           #replace text for each sp, else write a loop function
  loc.data = THCY,         #species df
  lat.col = "ddlat",
  long.col = "ddlon",
  spec.col = "species",
  thin.par = 1,             #dist in km to separate records
  reps = 5,                 #repetitions run
  locs.thinned.list.return = TRUE,
  write.files = TRUE,
  max.files = 5,
  out.dir = "THCY_thinned_full/",   #creates subdir
  out.base = "thinned_THCY",
  write.log.file = TRUE,
  log.file = "THCY_thin_log.txt"
)
tic()


#saving converged thinning dfs for downstream use
THCY_thin<-read.csv('./THCY_thinned_full/thinned_THCY_thin3.csv', stringsAsFactors = FALSE)
THRU_thin<-read.csv('./THRU_thinned_full/thinned_THRU_thin2.csv', stringsAsFactors = FALSE)
THEL_thin<-read.csv('./THEL_thinned_full/thinned_THEL_thin3.csv', stringsAsFactors = FALSE)
THEQ_thin<-read.csv('./THEQ_thinned_full/thinned_THEQ_thin2.csv', stringsAsFactors = FALSE)
THMA_thin<-read.csv('./THMA_thinned_full/thinned_THMA_thin2.csv', stringsAsFactors = FALSE)

THAM_thinned<-rbind(THCY_thin,THRU_thin,THEL_thin,THEQ_thin,THMA_thin) # 

#records after thinning to 1/km^2
table(THAM_thinned$species)
  # THCY THEL THEQ THMA THRU 
  # 127  114   20   58   28 
#
#
#########################################

#Optional: Get Future Data online [https://rdrr.io/github/rspatial/geodata/man/cmip6.html]

### !! If downloaded/stored separately, skip to next section line 219 ###

# library(remotes)
# remotes::install_github("rspatial/geodata")
#library(geodata)
#FutureClim_path<-".\\R\\THAM_SDM\\FutureClim\\"

#tic()
#GetFuturedata<-cmip6_world(                                                            ##very large files, recommended one at a time
  #"HadGEM3-GC31-LL", "126", "2041-2060", var="bioc", res=0.5, path=FutureClim_path #2260s
  #"HadGEM3-GC31-LL", "585", "2041-2060", var="bioc", res=0.5, path=FutureClim_path #2163s
  #"HadGEM3-GC31-LL", "126", "2081-2100", var="bioc", res=0.5, path=FutureClim_path #2391s
  #"HadGEM3-GC31-LL", "585", "2081-2100", var="bioc", res=0.5, path=FutureClim_path #2306s
  #"MPI-ESM1-2-LR", "126", "2041-2060", var="bioc", res=0.5, path=FutureClim_path # s
  #"MPI-ESM1-2-LR", "585", "2041-2060", var="bioc", res=0.5, path=FutureClim_path # s
  #"MPI-ESM1-2-LR", "126", "2081-2100", var="bioc", res=0.5, path=FutureClim_path # s
  #"MPI-ESM1-2-LR", "585", "2081-2100", var="bioc", res=0.5, path=FutureClim_path # s
  #"MRI-ESM2-0", "126", "2041-2060", var="bioc", res=0.5, path=FutureClim_path # 
  #"MRI-ESM2-0", "585", "2041-2060", var="bioc", res=0.5, path=FutureClim_path # 
  #"MRI-ESM2-0", "126", "2081-2100", var="bioc", res=0.5, path=FutureClim_path # s
  #"MRI-ESM2-0", "585", "2081-2100", var="bioc", res=0.5, path=FutureClim_path # s
#)
#toc() #2260s ~38min each ~a merged file that needs to be manually extracted per bioclim var i.e., 'band' 
  

#########################################
## Set up ENV data
##  bioclimatic variables (Worldclim v2, Bioclim)
## *note, stored locally in working directory

## Prepare current (i.e., 'present') time,
stk_current <- 
  rast(
    c(
      bio11 = "bio11_AZ.tif",
      bio15 = "Bio15_AZ1.tif",
      bio16 = "bio16_AZ.tif",
      bio17 = "bio17_AZ.tif",
      bio4 =  "Bio4_AZ1.tif",
      bio5 =  "Bio5_AZ1.tif",
      elev30 = "Elev30s_AZ.tif"
    ))
names(stk_current) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30") #must rename raster layers



## FUTURE 2050 (2041-2060) 

#climatic variables: HadGEM3 SSP126 
stk_2050_HAD_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/HadGEM3_ssp126_2050_bio11.tif",
      bio15 = "./FutureClim/HadGEM3_ssp126_2050_bio15.tif",
      bio16 = "./FutureClim/HadGEM3_ssp126_2050_bio16.tif",
      bio17 = "./FutureClim/HadGEM3_ssp126_2050_bio17.tif",
      bio4 =  "./FutureClim/HadGEM3_ssp126_2050_bio4.tif",
      bio5 =  "./FutureClim/HadGEM3_ssp126_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_HAD_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")


## 2050 climatic variables: HadGEM3 SSP585 (2041-2060) 
stk_2050_HAD_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/HadGEM3_ssp585_2050_bio11.tif",
      bio15 = "./FutureClim/HadGEM3_ssp585_2050_bio15.tif",
      bio16 = "./FutureClim/HadGEM3_ssp585_2050_bio16.tif",
      bio17 = "./FutureClim/HadGEM3_ssp585_2050_bio17.tif",
      bio4 =  "./FutureClim/HadGEM3_ssp585_2050_bio4.tif",
      bio5 =  "./FutureClim/HadGEM3_ssp585_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_HAD_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2050 climatic variables: MPI SSP126 (2041-2060) 
stk_2050_MPI_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/MPI_ssp126_2050_bio11.tif",
      bio15 = "./FutureClim/MPI_ssp126_2050_bio15.tif",
      bio16 = "./FutureClim/MPI_ssp126_2050_bio16.tif",
      bio17 = "./FutureClim/MPI_ssp126_2050_bio17.tif",
      bio4 =  "./FutureClim/MPI_ssp126_2050_bio4.tif",
      bio5 =  "./FutureClim/MPI_ssp126_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_MPI_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2050 climatic variables: MPI SSP585 (2041-2060) 
stk_2050_MPI_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/MPI_ssp585_2050_bio11.tif",
      bio15 = "./FutureClim/MPI_ssp585_2050_bio15.tif",
      bio16 = "./FutureClim/MPI_ssp585_2050_bio16.tif",
      bio17 = "./FutureClim/MPI_ssp585_2050_bio17.tif",
      bio4 =  "./FutureClim/MPI_ssp585_2050_bio4.tif",
      bio5 =  "./FutureClim/MPI_ssp585_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_MPI_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2050 climatic variables: MRI SSP126 (2041-2060) 
stk_2050_MRI_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/MRI_ssp126_2050_bio11.tif",
      bio15 = "./FutureClim/MRI_ssp126_2050_bio15.tif",
      bio16 = "./FutureClim/MRI_ssp126_2050_bio16.tif",
      bio17 = "./FutureClim/MRI_ssp126_2050_bio17.tif",
      bio4 =  "./FutureClim/MRI_ssp126_2050_bio4.tif",
      bio5 =  "./FutureClim/MRI_ssp126_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_MRI_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")


## 2050 climatic variables: MRI SSP126 (2041-2060) 
stk_2050_MRI_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/MRI_ssp585_2050_bio11.tif",
      bio15 = "./FutureClim/MRI_ssp585_2050_bio15.tif",
      bio16 = "./FutureClim/MRI_ssp585_2050_bio16.tif",
      bio17 = "./FutureClim/MRI_ssp585_2050_bio17.tif",
      bio4 =  "./FutureClim/MRI_ssp585_2050_bio4.tif",
      bio5 =  "./FutureClim/MRI_ssp585_2050_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2050_MRI_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## FUTURE 2090 (2081-2100) ##

#climatic variables: HadGEM3 SSP126 
stk_2090_HAD_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/HadGEM3_ssp126_2090_bio11.tif",
      bio15 = "./FutureClim/HadGEM3_ssp126_2090_bio15.tif",
      bio16 = "./FutureClim/HadGEM3_ssp126_2090_bio16.tif",
      bio17 = "./FutureClim/HadGEM3_ssp126_2090_bio17.tif",
      bio4 =  "./FutureClim/HadGEM3_ssp126_2090_bio4.tif",
      bio5 =  "./FutureClim/HadGEM3_ssp126_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_HAD_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")


## 2090 climatic variables: HadGEM3 SSP585 (2041-2060) 
stk_2090_HAD_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/HadGEM3_ssp585_2090_bio11.tif",
      bio15 = "./FutureClim/HadGEM3_ssp585_2090_bio15.tif",
      bio16 = "./FutureClim/HadGEM3_ssp585_2090_bio16.tif",
      bio17 = "./FutureClim/HadGEM3_ssp585_2090_bio17.tif",
      bio4 =  "./FutureClim/HadGEM3_ssp585_2090_bio4.tif",
      bio5 =  "./FutureClim/HadGEM3_ssp585_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_HAD_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2090 climatic variables: MPI SSP126 (2041-2060) 
stk_2090_MPI_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/MPI_ssp126_2090_bio11.tif",
      bio15 = "./FutureClim/MPI_ssp126_2090_bio15.tif",
      bio16 = "./FutureClim/MPI_ssp126_2090_bio16.tif",
      bio17 = "./FutureClim/MPI_ssp126_2090_bio17.tif",
      bio4 =  "./FutureClim/MPI_ssp126_2090_bio4.tif",
      bio5 =  "./FutureClim/MPI_ssp126_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_MPI_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2090 climatic variables: MPI SSP585 (2041-2060) 
stk_2090_MPI_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/MPI_ssp585_2090_bio11.tif",
      bio15 = "./FutureClim/MPI_ssp585_2090_bio15.tif",
      bio16 = "./FutureClim/MPI_ssp585_2090_bio16.tif",
      bio17 = "./FutureClim/MPI_ssp585_2090_bio17.tif",
      bio4 =  "./FutureClim/MPI_ssp585_2090_bio4.tif",
      bio5 =  "./FutureClim/MPI_ssp585_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_MPI_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")

## 2090 climatic variables: MRI SSP126 (2041-2060) 
stk_2090_MRI_126 <- 
  rast(
    c(
      bio11 = "./FutureClim/MRI_ssp126_2090_bio11.tif",
      bio15 = "./FutureClim/MRI_ssp126_2090_bio15.tif",
      bio16 = "./FutureClim/MRI_ssp126_2090_bio16.tif",
      bio17 = "./FutureClim/MRI_ssp126_2090_bio17.tif",
      bio4 =  "./FutureClim/MRI_ssp126_2090_bio4.tif",
      bio5 =  "./FutureClim/MRI_ssp126_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_MRI_126) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")


## 2090 climatic variables: MRI SSP126 (2041-2060) 
stk_2090_MRI_585 <- 
  rast(
    c(
      bio11 = "./FutureClim/MRI_ssp585_2090_bio11.tif",
      bio15 = "./FutureClim/MRI_ssp585_2090_bio15.tif",
      bio16 = "./FutureClim/MRI_ssp585_2090_bio16.tif",
      bio17 = "./FutureClim/MRI_ssp585_2090_bio17.tif",
      bio4 =  "./FutureClim/MRI_ssp585_2090_bio4.tif",
      bio5 =  "./FutureClim/MRI_ssp585_2090_bio5.tif",
      elev30 = "Elev30s_AZ.tif"
    )
  )
names(stk_2090_MRI_585) <- c("bio11","bio15","bio16","bio17","bio4","bio5","elev30")



####################################################
####################################################
## simple models with current env vars
## helpful in determining which to use in future projections
## get occurrences points
#

myRespName <- 'THCY_thinned' #change response name to desired (thinned) species df 

## formatting the data
myBiomodData <- 
  BIOMOD_FormatingData(
    resp.var = rep(1, nrow(THCY_thin)), #set desired species (thinned df)
    expl.var = unwrap(stk_current),
    resp.xy = THCY_thin[, c("ddlon", "ddlat")],
    resp.name = myRespName,
    PA.strategy = "random", 
    PA.nb.rep = 3,         #repetitions 
    PA.nb.absences =  1000 #number of pseudo absences 
  )

## print formatting summary
myBiomodData

# Create modeling options
myBiomodOptions <- BIOMOD_ModelingOptions(
  GAM = list(algo = "GAM_mgcv",method="REML"),
  GLM = list(type = 'quadratic',interaction.level = 0),
  RF = list(ntree=1000)
)

myBiomodOptions


#single model
tic()
myBiomodModelOut2 <- BIOMOD_Modeling(bm.format = myBiomodData,
                                     bm.options = myBiomodOptions,
                                     modeling.id = '3Algos',
                                     models = c("GBM", "GAM",'RF'),   #included  algorithms
                                     nb.rep = 3,                      #repetition of pres/abs
                                     data.split.perc = 80,            #training/testing split, value = training
                                     # data.split.table = myBiomodCV, #not run
                                     var.import = 3,                  #repetitions of variable importance score
                                     metric.eval = c('TSS','ROC'),    #evaluation metrics
                                     do.full.models = FALSE,
                                     nb.cpu = 6)
toc() #~135-770s to run

myBiomodModelOut2

# Get evaluation scores & variables importance
get_evaluations(myBiomodModelOut2)
THAM_single_eval<-get_evaluations(myBiomodModelOut2)%>%
  group_by(algo)%>%
  summarise(mean.sens=mean(sensitivity),
            mean.spec=mean(specificity),
            mean.cal=mean(calibration),
            mean.val=mean(validation),
            .groups = 'drop') %>%
  as.data.frame()
THAM_single_eval
#   algo mean.sens mean.spec  mean.cal  mean.val   {sens = true pos rate; spec = true neg rate}


get_variables_importance(myBiomodModelOut2)

# Viz evaluation scores & variables importance
bm_PlotEvalMean(bm.out = myBiomodModelOut2) #plots algos by ROC x TSS scores
bm_PlotEvalMean(bm.out = myBiomodModelOut2, dataset = 'calibration')
bm_PlotEvalMean(bm.out = myBiomodModelOut2, dataset = 'validation')
bm_PlotEvalBoxplot(bm.out = myBiomodModelOut2, group.by = c('algo', 'algo'))
bm_PlotEvalBoxplot(bm.out = myBiomodModelOut2, group.by = c('algo', 'run'))
bm_PlotVarImpBoxplot(bm.out = myBiomodModelOut2, group.by = c('expl.var', 'algo', 'algo'))
bm_PlotVarImpBoxplot(bm.out = myBiomodModelOut2, group.by = c('expl.var', 'run', 'algo'))
bm_PlotVarImpBoxplot(bm.out = myBiomodModelOut2, group.by = c('run', 'expl.var', 'algo'))
#
#  
#######################
## Project single models 
tic()
myBiomodProj <- BIOMOD_Projection(bm.mod = myBiomodModelOut2,
                                  proj.name = 'Current',
                                  new.env = stk_current,
                                  models.chosen = 'all',
                                  metric.binary = 'all',
                                  metric.filter = 'all',
                                  build.clamping.mask = TRUE,
                                  nb.cpu = 6)
toc() #428s
myBiomodProj

#######################
## Create Ensemble SDMs
#
tic()
myBiomodEM <- BIOMOD_EnsembleModeling(bm.mod = myBiomodModelOut2,
                                      models.chosen = 'all',  #selection from output models
                                      em.by = 'all',          #PA+run, PA+algo, all
                                      metric.select = c('TSS'), #eval metric to pair with threshold for individual models
                                      metric.select.thresh = c(0.70), #min score to to exclude individual models from the ensemble build 
                                      var.import = 3,                 #repetitions for var import scores
                                      metric.eval = c('TSS', 'ROC'),  #eval metrics, i.e., from final ensemble
                                      em.algo = c('EMci', 'EMca', 'EMwmean'), #conf int, committee averaging, weighted mean
                                      EMci.alpha = 0.05, #i.e., sig level for CI
                                      EMwmean.decay = 'proportional', #relative importance of weights..."proportionally" to the modl eval score
                                      nb.cpu = 6) 
toc() #125-290s
myBiomodEM

# Get evaluation scores & variables importance
get_evaluations(myBiomodEM) #want values close to 1  

#get_variables_importance(myBiomodEM)
THAMvar.importance<-get_variables_importance(myBiomodEM)%>%
  group_by(expl.var)%>%
  summarise(mean.imp=mean(var.imp),.groups = 'drop') %>%
  as.data.frame()
THAMvar.importance
#write.table(THAMvar.importance, file = paste0("THCYvar.importance", ".csv")) #saves to main dir


# Represent evaluation scores & variables importance
bm_PlotEvalMean(bm.out = myBiomodEM, group.by = 'full.name') #
bm_PlotEvalBoxplot(bm.out = myBiomodEM, group.by = c('full.name', 'full.name')) #
bm_PlotVarImpBoxplot(bm.out = myBiomodEM, group.by = c('expl.var', 'full.name', 'full.name'))
bm_PlotVarImpBoxplot(bm.out = myBiomodEM, group.by = c('expl.var', 'algo', 'merged.by.run')) #var importance plot
bm_PlotVarImpBoxplot(bm.out = myBiomodEM, group.by = c('algo', 'expl.var', 'merged.by.run'))

###
# Project ensemble models (from single projections)
# "BIOMOD_EnsembleForecasting" is analogous to "BIOMOD_Projection"
tic()
myBiomodEMProj <- BIOMOD_EnsembleForecasting(bm.em = myBiomodEM, 
                                             bm.proj = myBiomodProj,
                                             models.chosen = 'all',
                                             metric.binary = 'all',
                                             metric.filter = 'all')
toc() #6s

# Project ensemble models (building single projections)
tic()
myBiomodEMProj <- BIOMOD_EnsembleForecasting(bm.em = myBiomodEM,
                                             proj.name = 'CurrentEM',
                                             new.env = stk_current,
                                             models.chosen = 'all',
                                             metric.binary = 'all',
                                             metric.filter = 'all',
                                             nb.cpu=6)
toc() #~144s
myBiomodEMProj

#panel plot of ensembles (ca,wmean) 
THCY_EM_current<-plot(myBiomodEMProj)+
  theme(axis.text.x = element_text(angle = 45, vjust = 0.25, hjust=0.5))+ #
  theme(strip.background = element_blank())+ #removes background grey 
  theme(panel.spacing.x = unit(1.25,"lines"))+ #adds some space between plots
  theme(strip.text = element_text(colour = "white")) # hide the text
THCY_EM_current  


###
# Future Projection Scenarios
proj_scen <- c("current",
               "2050_HAD_126", "2050_HAD_585","2050_MPI_126", "2050_MPI_585","2050_MRI_126", "2050_MRI_585",
               "2090_HAD_126", "2090_HAD_585","2090_MPI_126", "2090_MPI_585","2090_MRI_126", "2090_MRI_585"
)
#each above is unique SpatRaster related to "period x GCM x SSP" 

tic() #tictoc for quick runtime estimates
for(scen in proj_scen){
  cat("\n> projections of ", scen) #loops for each scen
  
  FutureEM<-BIOMOD_EnsembleForecasting(
    bm.em=myBiomodEM,
    #bm.proj = myBiomodEMProj,
    proj.name = paste0('FutureEM_', scen),
    new.env = get(paste0("stk_", scen)),
    new.env.xy = NULL,
    models.chosen = "all",
    metric.binary = 'TSS',
    metric.filter = NULL,
    compress = TRUE,
    nb.cpu = 6,
  )
  
  #save plot per scen
  pdf(paste0("Graphic_FutureEM_", scen, ".pdf"))
  plot(FutureEM)+ #similar panel plot to current but for future scen (order: ca, wmean)
    theme(axis.text.x = element_text(angle = 45, vjust = 0.25, hjust=0.5))+ #works
    theme(strip.background = element_blank())+ #removes background grey but not test
    theme(panel.spacing.x = unit(1.25,"lines"))+ #adds some space between plots
    theme(strip.text = element_text(colour = "white")) # hide the text
  dev.off() 
  
  # Load current and future binary projections per scen
  CurrentProjEM <- get_predictions(myBiomodEMProj, metric.binary = "TSS")
  FutureProjEM <- get_predictions(FutureEM, metric.binary = "TSS")
  
  # Compute differences per scen
  myBiomodRangeSizeEM <- BIOMOD_RangeSize(proj.current = CurrentProjEM, 
                                          proj.future = FutureProjEM)
  
  rangeEM_comp <- myBiomodRangeSizeEM$Compt.By.Models
  rangeEM_map <- myBiomodRangeSizeEM$Diff.By.Pixel
  writeRaster(rangeEM_map, filename = paste0("rangeEM_map_", scen, ".tif"),overwrite=TRUE)
  write.table(rangeEM_comp, file = paste0("rangeEM_comp_", scen, ".csv")) #table of range change values (pixels) and %
  
  pdf(paste0("Graphic_RangeEM_", scen, ".pdf"))
  plot(rangeEM_map)+ #2-panel plot of ca vs wmean for range changes (4 levels)
    theme(axis.text.x = element_text(angle = 45, vjust = 0.25, hjust=0.5))+ #works
    theme(strip.background = element_blank())+ #removes background grey but not test
    theme(panel.spacing.x = unit(1.25,"lines"))+ #adds some space between plots
    theme(strip.text = element_text(colour = "white")) #hide the text
  dev.off()
}
toc() #~2122-4420s(35-74m)  

save.image("THCY_separate_full_run") # ~full pipeline with separate files per 'period_GCM_ssp'

### Cycle back to line 430, adjust sp on lines [436,441,443,548,582,587,647], and repeat through here for next species ###
######################################################################

###
### EM evaluation and stats ###

#load datasets
THAM.EM.eval<-read.csv("THAM_EM_eval.csv",sep=",", header=T) #upload ensemble model evaluation df

THAM.EM.eval.cols <- c("species","EM_algo","met","var")  
THAM.EM.eval[,THAM.EM.eval.cols] <- lapply(THAM.EM.eval[,THAM.EM.eval.cols], factor)
str(THAM.EM.eval)  

#Algo subset
Algo.tests<-THAM.EM.eval %>% dplyr::select(!c(name,var,var.imp))%>%na.omit() #temp exclude unnecessary vars here...

group_by(Algo.tests, met) %>% #assess eval metrics "met" or ensemble algorithms "EM_algo"
  summarise(
    count = n(),
    mean.sens = mean(sens, na.rm = TRUE),
    sd.sens = sd(sens, na.rm = TRUE),
    mean.spec = mean(spec, na.rm = TRUE),
    sd.spec = sd(spec, na.rm = TRUE),
    mean.cal = mean(cal, na.rm = TRUE),
    sd.cal = sd(cal, na.rm = TRUE)
  )
  #   EM_algo count mean.sens sd.sens mean.spec sd.spec mean.cal sd.cal
  # 1 ca         10      99.7   0.432      96.3    3.16    0.976 0.0307
  # 2 wmean      10      98.8   1.09       95.5    3.91    0.966 0.0431
  
  # 1 ROC        10      99.2   0.944      95.9    3.56    0.992 0.00775
  # 2 TSS        10      99.2   0.944      95.8    3.60    0.951 0.0430 

ggline(Algo.tests, x = "species", y = "cal",
       add = c("mean_se", "jitter"),
       #order = c("ctrl", "trt1", "trt2"),
       ylab = "calibration", xlab = "Thamnophis")

kruskal.test(cal ~ species, data = Algo.tests) #assess sens,spec,cal by met, EM_algo
pairwise.wilcox.test(Algo.tests$cal, Algo.tests$species,
                     p.adjust.method = "BH")
  #sens ~ EM_algo = Kruskal-Wallis chi-squared = 3.171, df = 1, p-value = 0.07495
  #spec ~ EM_algo = Kruskal-Wallis chi-squared = 1.1259, df = 1, p-value = 0.2886
  #cal ~ EM_algo = Kruskal-Wallis chi-squared = 0.51923, df = 1, p-value = 0.4712
  #sens ~ met = Kruskal-Wallis chi-squared = 0, df = 1, p-value = 1
  #spec ~ met = Kruskal-Wallis chi-squared = 0.012925, df = 1, p-value = 0.9095
  #cal ~ met = Kruskal-Wallis chi-squared = 3.741, df = 1, p-value = 0.05309

#var imp subset
var.import.tests<-THAM.EM.eval %>% dplyr::select(species,var,var.imp)%>%na.omit() #select only var imp data
levels(var.import.tests$species)<-list("Thamnophis cyrtopsis"="THCY", 
                                       "Thamnophis elegans"="THEL", 
                                       "Thamnophis eques"="THEQ",
                                       "Thamnophis marcianus"="THMA",
                                       "Thamnophis rufipunctatus"="THRU")
var.import.tests$var = factor(var.import.tests$var,
                              ordered = FALSE,
                              levels=c("bio4", "bio5", "bio11","bio15","bio16","bio17","elev30"),
                              labels=c("bio4", "bio5", "bio11","bio15","bio16","bio17","elev"))

group_by(var.import.tests, var) %>% #assess by bioclimatic variable "var" and species
  summarise(
    count = n(),
    mean.var.imp = mean(var.imp, na.rm = TRUE),
    sd.var.imp = sd(var.imp, na.rm = TRUE)
  )

#
kruskal.test(var.imp ~ var, data = var.import.tests)
  #var.imp ~ species = Kruskal-Wallis chi-squared = 4.5741, df = 4, p-value = 0.3338
  #var.imp ~ var = Kruskal-Wallis chi-squared = 5.1276, df = 6, p-value = 0.5276

#Plot for variable importance scores by species
ggplot(var.import.tests) +
  geom_col(aes(x = var, y = var.imp, fill = species), position = "stack")+
  geom_text(aes(x = var, y = var.imp,label = round(var.imp, 3),group=species), color="white",position = position_stack(vjust = 0.8))+
  ylab("Variable importance")+
  xlab("Predictor")+
  scale_fill_manual(values=cbPalette)


#by species if desired..
ggplot(var.import.tests) +
  geom_col(aes(x = species, y = var.imp, fill = var), position = "stack")+
  geom_text(aes(x = species, y = var.imp,label = round(var.imp, 3),group=var), color="white",position = position_stack(vjust = 0.8))+
  ylab("Variable importance")+
  xlab("Species")+
  scale_fill_manual(values=cbPalette)



#####################################################
###### scenario evaluation and stats
######
THAM.scen.eval<-read.csv("THAM_scen_eval.csv",sep=",", header=T)

THAM.scen.eval.cols <- c("Period",	"GCM",	"SSP","Per.SSP",	"Species",	"Emalgo")  
THAM.scen.eval[,THAM.scen.eval.cols] <- lapply(THAM.scen.eval[,THAM.scen.eval.cols], factor)

THAM.scen.eval$Period = factor(THAM.scen.eval$Period,
                               ordered = FALSE,
                               levels=c("current","2050","2090"),
                               labels=c("current","2050","2090"))
str(THAM.scen.eval) 

#omit 'current' from df
THAM.scen.eval<-dplyr::filter(THAM.scen.eval,GCM!="current")

#descriptive stats for scenario outcomes
group_by(THAM.scen.eval, Species,Emalgo) %>%
  summarise(
    count = n(),
    mean.current = mean(CurrentRangeSize, na.rm = TRUE),
    sd.current = sd(CurrentRangeSize, na.rm = TRUE),
    mean.change.perc = mean(SpeciesRangeChange, na.rm = TRUE),
    sd.change.perc = sd(SpeciesRangeChange, na.rm = TRUE)
  )

#Kruskal-Wallis looped over numeric data by factor Emalgo
scen_Per.SSP.results <- list()
for(i in names(THAM.scen.eval[,7:16])){  
  scen_Per.SSP.results[[i]] <- kruskal.test(formula(paste(i, "~ Per.SSP")), data = THAM.scen.eval) # "Per.SSP"
}
scen_Per.SSP.results #prints the list
    #Loss: Kruskal-Wallis chi-squared = 1.645, df = 3, p-value = 0.6492
    #Stable0: Kruskal-Wallis chi-squared = 0.091022, df = 3, p-value = 0.9929
    #Stable1: Kruskal-Wallis chi-squared = 4.8595, df = 3, p-value = 0.1824
    #Gain: Kruskal-Wallis chi-squared = 3.7863, df = 3, p-value = 0.2855
    #PercLoss: Kruskal-Wallis chi-squared = 10.966, df = 3, p-value = 0.01191 *
    #PercGain: Kruskal-Wallis chi-squared = 3.914, df = 3, p-value = 0.2709
    #SpeciesRangeChange: Kruskal-Wallis chi-squared = 7.8446, df = 3, p-value = 0.04934 *
    #CurrentRangeSize: Kruskal-Wallis chi-squared = 0, df = 3, p-value = 1
    #FutureRangeSize.NoDisp: Kruskal-Wallis chi-squared = 4.8595, df = 3, p-value = 0.1824
    #FutureRangeSize.FullDisp: Kruskal-Wallis chi-squared = 2.1798, df = 3, p-value = 0.5359


#Visualize n=220 Projected range change% ensemble models per period x SSP x GCM
GCM.proj<-ggline(THAM.scen.eval, x = "GCM", y = "SpeciesRangeChange",group="Per.SSP",color="Per.SSP",
                 add = c("mean_se", "jitter"),palette=cbPalette,
                 #order = c("HAD", "MPI","MRI"),
                 ylab = "Range change (%)", xlab = "Global Climate Models")
GCM.proj<-GCM.proj+scale_x_discrete(labels=c("HAD" = "HadGEM3-GC31-LL", 
                                             "MPI" = "MPI-ESM1-2-LR",
                                             "MRI" = "MRI-ESM2-0"))+
  theme(axis.text.x = element_text(color = "grey20", size = 10))+
  theme(axis.text.y = element_text(color = "grey20", size = 10))
GCM.proj<-ggpar(GCM.proj,legend.title = "Projected pathways")+theme_pubclean()
GCM.proj


#use weighted mean ensemble algorithm only (statistically equivalent to "ca" but more often reported in lit)
wmean.only<-dplyr::filter(THAM.scen.eval,Emalgo=="wmean")%>% as.data.frame() #only using the wmean algo i.e., omitting ca

#get stats for range change
wmean.only<-wmean.only%>%
  group_by(Species,Per.SSP) %>%
  summarise(
    count = n(),
    mean.Loss = mean(Loss, na.rm = TRUE),
    sd.Loss = sd(Loss, na.rm = TRUE),
    se.Loss = sd.Loss/sqrt(count),
    mean.change.perc = mean(SpeciesRangeChange, na.rm = TRUE),
    sd.change.perc = sd(SpeciesRangeChange, na.rm = TRUE),
    se.change.perc = sd.change.perc/sqrt(count),
    .groups = 'drop'
  )%>% as.data.frame() #ungroup()

#Plot range change by species
range.proj<-ggplot(wmean.only )+
  geom_bar(aes(x= Species,y=mean.change.perc,fill=Per.SSP),stat="identity", color="black", position=position_dodge()) +
  geom_errorbar(aes(x= Species,ymin=mean.change.perc-se.change.perc, ymax=mean.change.perc+se.change.perc,fill=Per.SSP), 
                width=.2,  position=position_dodge(.9))+
  scale_x_discrete(name = "", labels = expression(italic("Thamnophis cyrtopsis"), 
                                                  italic("Thamnophis elegans"), 
                                                  italic("Thamnophis eques"),
                                                  italic("Thamnophis marcianus"),
                                                  italic("Thamnophis rufipunctatus")))+
  scale_fill_manual(name = "Projected pathway",values = cbPalette)+
  ggpubr::theme_pubclean()+
  theme(axis.title.x = element_blank(), legend.position = c(0.1,0.9))+
  coord_cartesian(expand = FALSE)+
  scale_y_continuous(name = expression("Range change (%)"),breaks = seq(-120,140,20))
range.proj


#
#
####################################################
### END ###


